; /**
;   ******************************************************************************
;   *  _ _ _    _ _ _   _ _
;   * |     \  /     |/     \
;   * |      \/           _ _\
;   * |                  /
;   * |         MultiClet
;   * |                  \ _ _
;   * |                      /
;   * |_ _ _|\/|_ _ _|\ _ _ /
;   *
;   * @file    gptimer.asm 
;   * @author  Команда разработчиков Мультиклет
;   * @date    15-07-2015
;   * @brief   Пример работы с таймером общего назначения (GPTIMER) и интерфейсом UART. 
;   *          
;   ******************************************************************************
;   * @attention
;   *
;   * <h2><center>&copy; COPYRIGHT(c) 2015 Multiclet corp.</center></h2>
;   *
;   *  ДАННЫЙ ПРИМЕР НАПИСАН ДЛЯ БЫСТРОГО ОЗНАКОМЛЕНИЯ ПОЛЬЗОВАТЕЛЯ С
;   *  ВОЗМОЖНОСТЯМИ МУЛЬТИКЛЕТОЧНОГО ПРОЦЕССОРА. КОМПАНИЯ МУЛЬТИКЛЕТ НЕ НЕСЁТ
;   *  ОТВЕТСТВЕННОСТИ ЗА ВОЗМОЖНЫЙ ПРЯМОЙ ИЛИ КОСВЕННЫЙ УЩЕРБ, ПРИЧИНЁННЫЙ 
;   *  ИСПОЛЬЗОВАНИЕМ В ПОЛЬЗОВАТЕЛЬСКИХ ПРОДУКТАХ ИНФОРМАЦИИ, СОДЕРЖАЩЕЙСЯ
;   *  В ДАННОМ ПРИМЕРЕ.
;   * 
;   ******************************************************************************
;   */
;файл констант периферии для ассемблера
.include "R1_pcf.inc"

.alias UART_BDR UART3_BDR
.alias UART_CR UART3_CR
.alias UART_ST UART3_ST
.alias UART_DATA UART3_DATA

.text

;расположение светодиодов: 
;LED0 - GPIOB(1<<25), LED1 - GPIOB(1<<24), LED2 - GPIOB(1<<23)
;LED3 - GPIOB(1<<27), LED4 - GPIOC(1<<26), LED5 - GPIOB(1<<29)
;LED6 - GPIOB(1<<28), LED7 - GPIOA(1<<25)
;расположение UART:
;UART3[TXD] - GPIOF(1<<14), UART3[RXD] - GPIOF(1<<13)
;UART1[TXD] - GPIOA(1<<23), UART1[RXD] - GPIOA(1<<22), UART1[CTS] - GPIOA(1<<24), UART1[RTS] - GPIOA(1<<25)
;UART2[TXD] - GPIOC(1<<11), UART2[RXD] - GPIOC(1<<10)
;UART0[TXD] - GPIOC(1<<22), UART0[RXD] - GPIOC(1<<21)
;регистры Таймера общего назначения:
;TIMn_PSCVAL - текущее значение предделителя таймера n, TIMx_PSCPER - период предделителя таймера n
;TIMn_CNTVALx - текущее значение таймера n для канала x, TIMn_CNTPERx - период таймера n для канала x 
;TIMn_CRx - регистр управления таймера n для канала x

;сконфигурируем порт для LED1 на выход и для UART3 на альтернативную функцию
config_port:
    rdl GPIOB_DIR 
    orl @1, (1<<24) 
    wrl @1, GPIOB_DIR ;настраиваем порт B на выход для вывода 24
    rdl GPIOF_BPS
    orl @1, (1<<14)|(1<<13)
    wrl @1, GPIOF_BPS ;включаем альтернативную функцию порта F для UART3
    jmp timer0_start
complete

;устанавливаем период, предделитель, разрешение прерываний и непрерывный режим работы Таймера 0
timer0_start:
    getl 0xFFFFF
    getl 0x4
    getl 0xD 
    wrl @3, TIM0_CNT0 ;записываем период Таймера 0
    wrl @3, TIM0_PSC ;записываем предделитель Таймера 0
    wrl @3, TIM0_CR0 ;записываем значение в регистр управления Таймера 0
    jmp your_code
complete

;параграфы с кодом, для которого проводится измерение
your_code:
    getl 0x1
    getl 0x2
    subl @1, @2
    addl @1, 2015
    getl 0x12345
    getl 0x12345
    addl @1,@2
    setl #5, @1
    jmp timer0_stop
complete

;считываем значение счетчика Таймера 0
timer0_stop:
    rdl TIM0_CNTVAL0
    getl 0xFFFFF
    subl @1, @2
    mull @1, 5
    setl #0, @1
    jmp uart_send_ron0
complete

;вывод РОНа 0 по uart 
uart_send_ron0:
    setl #1, 0 ;установим счетчик для вывода 64-х бит
    jmp uart_config
complete

;настройка UART3
uart_config:
    getl 0x1A ;значение для UART_BDR рассчитывается по формуле: BRDIV = Fsys/(Fuart*8-1)
    wrl @1, UART_BDR
    getl 0x3 ;разрешаем работу RX, TX
    wrl @1, UART_CR
    jmp uart_tx_ready
complete

uart_tx_ready:
    rdl UART_ST
    getl 1<<9
    andl @1, @2
    jne @1, uart_tx_ready
    je @2, uart_send_byte
complete

uart_send_byte:
        getq #0
cnt:=   getl #1 ;запись cnt:= использована для удобства ссылки
        slrq @2, @1 ;сдвигаем вправо по счетчику
        wrb @1, UART_DATA
        addl @cnt, 8 ;увеличиваем счетчик на 8
        setl #1, @1 ;записываем значение в РОН 1
        subl @cnt, 56 ;проверяем отправку последнего байта
        je @1, led1_on
        jne @2, uart_tx_ready
complete

;включение светодиода
led1_on:
    rdl GPIOB_OUT ;считываем текущее значение порта в регистр передаваемых данных порта В
    orl @1, (1<<24) ;формируем значение для включения логической единицы на 24-м разряде порта B
    wrl @1, GPIOB_OUT ;записываем получившееся значение в регистр передаваемых данных порта B
complete




