; /**
;   ******************************************************************************
;   *  _ _ _    _ _ _   _ _
;   * |     \  /     |/     \
;   * |      \/           _ _\
;   * |                  /
;   * |         MultiClet
;   * |                  \ _ _
;   * |                      /
;   * |_ _ _|\/|_ _ _|\ _ _ /
;   *
;   * @file    pwm.asm 
;   * @author  Команда разработчиков Мультиклет
;   * @date    13-07-2015
;   * @brief   Пример работы с ШИМ и управления элементом BUZZER. 
;   *          
;   ******************************************************************************
;   * @attention
;   *
;   * <h2><center>&copy; COPYRIGHT(c) 2015 Multiclet corp.</center></h2>
;   *
;   *  ДАННЫЙ ПРИМЕР НАПИСАН ДЛЯ БЫСТРОГО ОЗНАКОМЛЕНИЯ ПОЛЬЗОВАТЕЛЯ С
;   *  ВОЗМОЖНОСТЯМИ МУЛЬТИКЛЕТОЧНОГО ПРОЦЕССОРА. КОМПАНИЯ МУЛЬТИКЛЕТ НЕ НЕСЁТ
;   *  ОТВЕТСТВЕННОСТИ ЗА ВОЗМОЖНЫЙ ПРЯМОЙ ИЛИ КОСВЕННЫЙ УЩЕРБ, ПРИЧИНЁННЫЙ 
;   *  ИСПОЛЬЗОВАНИЕМ В ПОЛЬЗОВАТЕЛЬСКИХ ПРОДУКТАХ ИНФОРМАЦИИ, СОДЕРЖАЩЕЙСЯ
;   *  В ДАННОМ ПРИМЕРЕ.
;   * 
;   ******************************************************************************
;   */
;файл констант периферии для ассемблера
.include "R1_pcf.inc"

.text

;расположение светодиодов: 
;LED0 - GPIOB(1<<25), LED1 - GPIOB(1<<24), LED2 - GPIOB(1<<23)
;LED3 - GPIOB(1<<27), LED4 - GPIOC(1<<26), LED5 - GPIOB(1<<29)
;LED6 - GPIOB(1<<28), LED7 - GPIOA(1<<25)

;регистры ШИМ (PWM)
;PWM_CR - регистр управления 
;PWM_INT - регистр прерываний
;PWM_CNTVAL - регистр текущего значения счетчика
;PWM_PSC - регистр предделителя
;PWM_CNT - регистр периода счетчика
;PWM_CMPCH0...PWM_CMPCH3 - регистр сравнения счетчика

; расположение ШИМ:
; канал0 - GPIOA(1<<26), канал1 - GPIOA(1<<25), канал2 - GPIOA(1<<24)
; канал3 - GPIOA(1<<23) 

;инициализируем параметры ШИМ
pwm_init:
    getl 0x0000FFFF
    wrl @1, PWM_CNT ;устанавливаем значение периода
    getl 0x00007FFF
    wrl @1, PWM_CMPCH0 ;устанавливаем значение регистра сравнения счетчика канала 0
    getl 0x00000010
    wrl @1, PWM_PSC ;устанавливаем предделитель
    jmp config_port
complete

;сконфигурируем порт B для LED2 на выход, а порт E для кнопки SW2 и порт C для кнопки SW3 на вход 
config_port:
    rdl GPIOB_DIR ;считываем текущее значение конфигурации порта B на вход и выход
    orl @1, (1<<23) 
    wrl @1, GPIOB_DIR 
    rdl GPIOE_DIR 
    andl @1, ~(1<<26) ;формируем значение для включения 26-го разряда порта E на вход
    wrl @1, GPIOE_DIR 
    rdl GPIOC_DIR
    andl @1, ~(1<<8) ;формируем значение для включения 8-го разряда порта С на вход
    wrl @1, GPIOC_DIR 
    rdl GPIOA_BPS ;считываем текущее значение конфигурации регистра альтернативных функции порта А
    orl @1, 1<<26 ;формируем значение для включения альтернативной функции 0-го канала ШИМ на порту А
    wrl @1, GPIOA_BPS ;записываем значение в регистр альтернативных функций порта А
    jmp wait_sw2 ;осуществляем переход на параграф с анализом кнопки SW2
complete

;анализ кнопки (без устранения дребезга кнопки)
wait_sw2:
    rdl GPIOE_IN ;получаем текущее значение порта E на вход
    andl @1, (1<<26) ;выделяем 26-й разряд
    je @1, led1_on ;если кнопка нажата, то зажигаем светодиод
    jne @2, wait_sw2 ;если кнопка не нажата, то остаемся в этом параграфе
complete

;включение светодиода
led1_on:
    rdl GPIOB_OUT ;считываем текущее значение порта в регистр передаваемых данных порта В
    orl @1, (1<<23) ;формируем значение для включения логической единицы на 23-м разряде порта B
    wrl @1, GPIOB_OUT ;записываем получившееся значение в регистр передаваемых данных порта B
    jmp pwm_start
complete

;включение ШИМ
pwm_start:
    getl 0x00100106 
    wrl @1, PWM_CR
    jmp wait_sw3 ;осуществляем переход на параграф с задержкой
complete

;анализ кнопки (без устранения дребезга кнопки)
wait_sw3:
    rdl GPIOC_IN ;получаем текущее значение порта E на вход
    andl @1, (1<<8) ;выделяем 8-й разряд
    je @1, led1_off ;если кнопка нажата, то гасим светодиод
    jne @2, wait_sw3 ;если кнопка не нажата, то остаемся в этом параграфе
complete

;выключение светодиода
led1_off:
    rdl GPIOB_OUT ;считываем текущее значение порта в регистр передаваемых данных порта В
    andl @1, ~(1<<23) ;формируем значение для выключения логической единицы на 23-м разряде порта B
    wrl @1, GPIOB_OUT ;записываем получившееся значение в регистр передаваемых данных порта B
    jmp pwm_stop ;осуществляем переход на параграф с задержкой
complete

;выключение ШИМ
pwm_stop:
    getl 0x00100006
    wrl @1, PWM_CR
    jmp wait_sw2 ;осуществляем переход на параграф с задержкой
complete





  


