/*
   ITU-T G.729 Annex C - Reference C code for floating point
                         implementation of G.729 Annex A
                         Version 1.01 of 15.September.98
*/

/*
----------------------------------------------------------------------
                    COPYRIGHT NOTICE
----------------------------------------------------------------------
   ITU-T G.729 Annex C ANSI C source code
   Copyright (C) 1998, AT&T, France Telecom, NTT, University of
   Sherbrooke.  All rights reserved.

----------------------------------------------------------------------
*/

/*-----------------------------------------------------------------*
 * Main program of the G.729a 8.0 kbit/s decoder.                  *
 *                                                                 *
 *    Usage : decoder  bitstream_file  synth_file                  *
 *                                                                 *
 *-----------------------------------------------------------------*/

#include "multiclet.h"
#include "TYPEDEF.h"
#include "ld8a.h"

int bad_lsf;        /* bad LSF indicator   */
/*
   This variable should be always set to zero unless transmission errors
   in LSP indices are detected.
   This variable is useful if the channel coding designer decides to
   perform error checking on these important parameters. If an error is
   detected on the  LSP indices, the corresponding flag is
   set to 1 signalling to the decoder to perform parameter substitution.
   (The flags should be set back to 0 for correct transmission).
*/

   FLOAT  synth_buf[L_FRAME+M];     /* Synthesis                  */
   FLOAT  *synth;   
   /*INT16 sp16[L_FRAME];*/
   extern FLOAT dec_past_qua_en[4];
   extern int DEBUG;

void decoder_init(){
    int i;
    /*-----------------------------------------------------------------*
 *           Initialization of decoder                             *
 *-----------------------------------------------------------------*/

  for (i=0; i<L_FRAME+M/*M*/; i++) synth_buf[i] = (F)0.0;
  synth = synth_buf + M;
  dec_past_qua_en[0]=(F)-14.0;
  dec_past_qua_en[1]=(F)-14.0;
  dec_past_qua_en[2]=(F)-14.0;
  dec_past_qua_en[3]=(F)-14.0;

  bad_lsf = 0;          /* Initialize bad LSF indicator */
  init_decod_ld8a();
  init_post_filter();
  init_post_process();
}

/*-----------------------------------------------------------------*
 *            Main decoder routine                                 *
 *-----------------------------------------------------------------*/

void decoder_main(uint16_t * serial,int16_t * sp16)
{
   static int parm[PRM_SIZE+2];            /* Synthesis parameters + BFI */
   /*static INT16 serial[SERIAL_SIZE];*/       /* Serial stream              */
   static FLOAT  Az_dec[MP1*2];            /* Decoded Az for post-filter */
   static int T2[2];                       /* Decoded Pitch              */

   int i;


/*-----------------------------------------------------------------*
 *            Loop for each "L_FRAME" speech data                  *
 *-----------------------------------------------------------------*/

      
      parm[1] =serial[0]&0xff;
      parm[2] =serial[1]&0x3ff;
      parm[3] =(serial[0]>>8)&0xff;
      parm[4] =(serial[1]>>10)&0x1;
      parm[5] =serial[2]&0x1fff;
      parm[6] =((serial[2]>>13)&0x7)+((serial[4]>>11)&0x8);
      parm[7] =serial[4]&0x7f;
      parm[8] =(serial[1]>>11)&0x1f;
      parm[9] =serial[3]&0x1fff;
      parm[10]=((serial[3]>>13)&0x7)+((serial[4]>>12)&0x8);
      parm[11]=(serial[4]>>7)&0x7f;
      

      /*if (serial[0] == SYNC_WORD) {*/
         parm[0] = 0;           /* No frame erasure */
      /*} else {
         parm[0] = 1;           /* frame erased     * /
      } */

      parm[4] = check_parity_pitch(parm[3], parm[4] ); /* get parity check result */
      
      decod_ld8a(parm, synth, Az_dec, T2);    

      post_filter(synth, Az_dec, T2);                  /* Post-filter */
      
      post_process(synth, L_FRAME);                    /* Highpass filter */

      fwrite16(synth, L_FRAME, sp16);
}
