/*
   ITU-T G.729 Annex C - Reference C code for floating point
                         implementation of G.729
                         Version 1.01 of 15.September.98
*/

/*
----------------------------------------------------------------------
                    COPYRIGHT NOTICE
----------------------------------------------------------------------
   ITU-T G.729 Annex C ANSI C source code
   Copyright (C) 1998, AT&T, France Telecom, NTT, University of
   Sherbrooke.  All rights reserved.

----------------------------------------------------------------------
*/

/*
 File : PRE_PROC.C
 Used for the floating point version of both
 G.729 main body and G.729A
*/


/*------------------------------------------------------------------------*
 * Function pre_process()                                                 *
 *                                                                        *
 * Preprocessing of input speech.                                         *
 *   - 2nd order high pass filter with cut off frequency at 140 Hz.       *
 *-----------------------------------------------------------------------*/

#include "multiclet.h"
#include "TYPEDEF.h"
#include "version.h"
#ifdef VER_G729A
 #include "ld8a.h"
 #include "tab_ld8a.h"
#else
 #include "ld8k.h"
 #include "tab_ld8k.h"
#endif

/*------------------------------------------------------------------------*
 * 2nd order high pass filter with cut off frequency at 140 Hz.           *
 * Designed with SPPACK efi command -40 dB att, 0.25 ri.                  *
 *                                                                        *
 * Algorithm:                                                             *
 *                                                                        *
 *  y[i] = b[0]*x[i] + b[1]*x[i-1] + b[2]*x[i-2]                          *
 *                   + a[1]*y[i-1] + a[2]*y[i-2];                         *
 *                                                                        *
 *     b[3] = {0.92727435E+00, -0.18544941E+01, 0.92727435E+00};          *
 *     a[3] = {0.10000000E+01, 0.19059465E+01, -0.91140240E+00};          *
 *-----------------------------------------------------------------------*/

#ifndef R1_o
static FLOAT px0, px1;         /* high-pass fir memory          */
static FLOAT py1, py2;         /* high-pass iir memory          */
#else
FLOAT px0, px1;         /* high-pass fir memory          */
FLOAT py1, py2;         /* high-pass iir memory          */
#endif

void init_pre_process( void
)
{
  px0 = px1 = (F)0.0;
  py2 = py1 = (F)0.0;

  return;
}
#ifndef R1_o
void pre_process(
   FLOAT signal[],      /* (i/o)  : signal                     */
   int lg               /* (i)    : lenght of signal           */
)
{
  int i;
  FLOAT x2;
  FLOAT y0;	 
  
  //printf("pre_process lg = 0x%X\n", lg);   

  for(i=0; i<lg; i++)
  {
    
    //printf("start itter pre_process i = 0x%X x1 = 0x%X x0 = 0x%X  signal[i] = 0x%X \n",i, x1,x0,signal[i]);   
    x2 = px1;
    px1 = px0;
    px0 = signal[i];
#ifndef R1_1
    y0 = py1*a140[1] + py2*a140[2] + px0*b140[0] + px1*b140[1] + x2*b140[2];
#else
	y0 = py1*a140[1] + py2*a140[2];
	y0 +=px0*b140[0] + px1*b140[1];
	y0 +=x2*b140[2];
#endif

    signal[i] = y0;
    py2 = py1;
    py1 = y0;
    //printf("end itter pre_process i = 0x%X y2 = 0x%X y1 = 0x%X  \n",i, y2,y1);  
    
  }

  return;
}
#endif
