/*
 Пример работы с ADC0 канал CH2.
 Соединить 2й вывод X5 субмодуля процессора и 2й (или 3й) вывод X14 на материнской плате.
 Соединить 9й (или 10й) вывод X24 и 5й (или 6й) вывод X14 на материнской плате. 
 Частота дискретизации 48 кГц.
 Усиление BG = 0 дБ.
 Усиление PGA = -25.5 + 17*1,5 = 0 дБ.
 
 */ 

#include <R1_ccf.h>
#include <gpio.h>
#include <adc.h>
#include <dac.h>
#include <tim.h>
#include <irq.h>

#define IRQ_VECTOR_GPTIM0 29
#define IRQ_VECTOR_ADC0 16


void interrupt_GPTIM0();
void interrupt_ADC0();


void main(){
    /*Устанавливаем тактовую частоту 80 МГц*/
    mc_set_pll(0x3, 0x6, 0xb, 1);
   
    /*Устанавливаем обработчики прерывания для таймера 0 и АЦП*/ 
    set_irq_vector(interrupt_GPTIM0, IRQ_VECTOR_GPTIM0);
    set_irq_vector(interrupt_ADC0, IRQ_VECTOR_ADC0);

	/*Настраиваем таймер 0*/
	TIM0->PSCPER = 0x4;
	TIM0->CNT0 = 0x4B0/6;
    
    /*Настраиваем АЦП и ЦАП*/    

    ADC0->CR = 0x60800;// 0110 0000 1000 0000 0000 fsample=48 KGz
    ADC0->CH2 = 0x11000000;// 0001 0001 0000 0000_0000 0000 0000 0000 PGA,BG
    DAC0->CR = 0;
    DAC1->CR = 0;      
    
    /*Включаем прерывания от АЦП и таймера 0*/
    asm_setl_MSKR((1<<29)|(1<<16));
    
	while(1);
}

void interrupt_GPTIM0(){
	unsigned dac = (((ADC0->CH0)&0xFFFF));
	dac=(dac+0x8000)>>4;
	DAC0->DATA = dac;
	DAC1->DATA = dac;
}

void interrupt_ADC0(){
	/*Включаем таймер*/
	 TIM0->CR0 = 0xD;
}




